"use strict";
import {testUtils} from './util/testUtils';
import {Deeplink, deeplinkFiredFlag, deeplinkFiredUrl} from './util/deeplink';
import {Router} from './util/router';
import {debugLogger as log, extendedDebugLogger as logExtended} from './logger/log';
import {CheckoutApplePayApp} from './pages/checkoutApplePayApp';
import {initPdfShare} from './util/pdfToShareDialog';
import {energyEfficiencyPdfToBrowser} from "./util/energyEfficiencyPdfToBrowser";
import DeviceDetails from './util/deviceDetails';
import {BarcodeScanner} from "./util/barcodeScanner";
import {globalHrefClickHandler} from "./util/globalHrefClickHandler";
import AdjustHandler from "./util/adjustHandler";
import WindowHandler from "./util/windowHandler";
import Constants from "./constants";
import UserDetails from "./util/userDetails";
import CookieHandler from "./util/cookieHandler";
import CountryHandler from "./util/countryHandler";
import {Dialogs as NativeDialogs} from "./util/dialogs";
import {Backbutton} from "./util/backbutton";
import AppSettings from "./util/appSettings";
import FingerprintSensor from "./util/fingerprintSensor";
import {initFeedbackFormPage} from "./pages/feedback.page";
import {persistUserCredentialsFromUi} from "./authorization/persistUserCredentialsFromUi";
import {initOrderConfirmationPageNewCheckout} from "./pages/order.confirmation.page";


(function ($) {
/**
 * App Code starts here. Workflow:
 * app.initialize
 *      registerCallbacks
 *      bindEvents
 *          onDeviceReady
 *              init
 *                  handleFirstAppStartup
 *                  handleSubsequentAppStartups
 *                  handleAutologin
 *                  handleFingerprintLogin
 *                  ...
 **/
var app = {
    initialize: function () {
        app.bindEvents();
        // prevent wkWebView from enabling pinch-to-zoom:
        $('head').append('<meta name="viewport" content="user-scalable=no, initial-scale=1.0, maximum-scale=1.0, minimum-scale=1.0, width=device-width" />');
        $('a').css('-webkit-touch-callout', 'none');
    },

    /**
     * Used to bind events ondeviceReady and login/logout
     */
    bindEvents: function () {
        $(window).on('cp.logoutSuccess', app.onLogoutSuccess);
        document.addEventListener('deviceready', app.onDeviceReady, false);
        $(window).on('cp.loginFailed', app.onLoginFailed);
        $(window).on('cp.loginSuccessful', app.onLoginSuccessful);
        $('.main-header').find('.logout').click(app.onLogoutClicked);
    },

    /**
     * When user is not on app landing page, function sets callback on login form at user flyout to persist credentials in app settings
     */
    registerFetchLoginCredentialsCallback: function () {
        try{
            if (!Router.isAppLandingPage(window.location.href)) {
                logExtended.debug("CREDENTIALHANDLING", "Try to assign cb function persistUserCredentialsFromUi to window.login.setAppCallbackFunction")
                if (typeof window.login !== undefined && typeof window.login.setAppCallbackFunction !== undefined) {
                    window.login.setAppCallbackFunction(persistUserCredentialsFromUi);
                    logExtended.debug("CREDENTIALHANDLING", "Success: Assigned cb function persistUserCredentialsFromUi to window.login.setAppCallbackFunction")
                } else {
                    logExtended.debug("CREDENTIALHANDLING", "Error: window.login.setAppCallbackFunction not available", {"window.login" : window.login})
                }
            }
        } catch(ex){
            logExtended.debug(
                "CREDENTIALHANDLING",
                "Exception while trying to execute registerCallbacks and trying to assign window.login.setAppCallbackFunction: "+ex,
                {
                    "err" : Object.assign({}, ex),
                    "window.login" : window.login}
                )
        }
    },

    /**
     *  add(): Register a handler with a unique name. If ne name is already registered, the callback function will not be added (no duplicates possible)
     *
     *  remove(): Remove a handler identified by its name
     *
     *  get(); get all handler functions
     */
    loginFailedHandler: {
        handlers: {},
        add: function(name, handler) {
            if (typeof  handler === 'function') {
                this.handlers[name] = handler;
            }
        },
        remove: function (name) {
            delete this.handlers[name];
        },
        get: function() {
            return this.handlers;
        }
    },

    /**
     * Invalidates credentials in app settings when login failed
     */
    onLoginFailed: function () {
        log.debug('cp.loginFailed fired!');

        /* Invalidate credentials */
        UserDetails.getAppSettings(function (err, res) {
            if (!err) {
                var settings = res;
                settings.credentials.valid = false;
                log.debug('fired: onLoginFailed; credentials: ' + settings.credentials.valid);
                UserDetails.updateAppSettings(settings, function (err, res) {
                    if(err) {
                        log.error('onLoginFailed() unable to persist credentials. err -> ' + JSON.stringify(err));
                    } else {
                        log.debug('onLoginFailed(); credentials saved: ' + res.credentials.valid);
                    }
                });
            } else {
                log.error('onLoginFailed() unable to read app settings. err -> ' + JSON.stringify(err));
            }
        });

        /* Handle Analytics: Fingerprintlogin failed */
        if(!UserDetails.isUserLoggedIn() && isFlagFingerprintLoginAttemptPerformed()){
            window.ADB.trackAction("login.Fingerprint", {"status":"Fehlgeschlagen - Falsche Zugangsdaten"});
            setFlagFingerprintloginAttemptPerformed(false);
        }
        /* Handle error handlers and messages */
        var errorHandlers = app.loginFailedHandler.get();

        if (Object.keys(errorHandlers).length === 0) {
            NativeDialogs.showNotificationAlert('Login fehlgeschlagen', 'Eine Anmeldung ist mit diesen Benutzerdaten nicht möglich.', 'OK');
        } else {
            Object.keys(errorHandlers).forEach(function (handlerName) {
                log.debug("errorhandler: "+handlerName);
                if (typeof  errorHandlers[handlerName] === 'function') {
                    errorHandlers[handlerName]();
                    app.loginFailedHandler.remove(handlerName);
                }
            });
        }
    },
    /**
     *
     */
    onLogoutSuccess: function() {
        log.info('cp.logoutSuccess fired!'); //
    },

    /**
     * Executed after successful login
     */
    onLoginSuccessful: function() {
        /* Handling for handleAskForFingerprintDecisionDialog on checkout page, because there is no page reload */
        if(Router.isPageCheckoutAccount() || Router.isPageCheckoutNew()){
            UserDetails.getAppSettings(function (err, settings) {
                if(!err) {
                    handleAskForFingerprintDecisionDialog(settings);
                } else {
                    log.error('onLoginSuccessful(): Unable to read appSettings from native storage! err -> ' + JSON.stringify(err));
                }
            });
        }
    },
    /**
     * Disables autologin and ask for fingerprintdecision in app settings when user clicked logout.
     */
    onLogoutClicked: function() {
        app.autoLogin.disableForLoggedOutUser();
        log.info('onLogoutClicked()');
        UserDetails.getAppSettings(function (err, settings) {
            if(!err) {
                settings.preventAutoLoginForLoggedOutUser = true;
                settings.showDialogAskForFingerprintDecision = false;
                UserDetails.updateAppSettings(settings, function (err, res) {
                    if(err) {
                        log.error('onLogoutClicked(): Unable to persist app settings. err -> ' + JSON.stringify(err));
                    } else {
                        log.info('onLogoutClicked(): Persisted logout');
                    }
                });
            } else {
                log.error('onLogoutClicked(): Unable to read appSettings from native storage! err -> ' + JSON.stringify(err));
            }
        });
    },

    onDeviceReady: function () {
        log.info('Device is ready.');
        Backbutton.showBackButtonIos();
        AdjustHandler.init();
        WindowHandler.init();
        init();
    },

    autoLogin: {
        disableForLoggedOutUser: function () {
            window.localStorage.setItem('appEnableAutoLogin', 'false');
            // Prevents Automatic Fingerprint-Verification after Logout for the Session
            app.fingerprintLogin.setFingerprintLoginAttemptOnStartupPerformed(true);
        },
        enableForLoggedOutUser: function () {
            window.localStorage.setItem('appEnableAutoLogin', 'true');
        },
        isEnabledForLoggedOutUser: function () {
            var show = window.localStorage.getItem('appEnableAutoLogin');
            if (show === 'true' || show === null) {
                return true;
            } else {
                return false;
            }
        },

        setExpired: function(bool) {
            window.sessionStorage.setItem('autoLoginExpired', '' + bool);
            log.debug("setExpired(): " +bool);
        },

        isExpired: function() {
            var expired = window.sessionStorage.getItem('autoLoginExpired');
            return (expired === null || expired === 'true') ?  true : false;
        },

        showExpiredMessage: function(settings){
            if(settings.countForSessionTimeoutMessage===1 && settings.credentials.username && settings.credentials.password){
                return true;
            } else{
                return false;
            }
        },

        setShowFingerprintWhenExpired: function (bool) {
            window.sessionStorage.setItem('showFingerprintWhenExpired', '' + bool);
        },

        isShowFingerprintWhenExpired: function () {
            var show = window.sessionStorage.getItem('showFingerprintWhenExpired');
            return (show === null || show === 'true') ?  true : false;
        },
    },

    fingerprintLogin : {
        isAskForFingerprintDecision: function() {
            var ask = window.sessionStorage.getItem('askForFingerprint');
            return (ask === null || ask === 'true') ?  true : false;
        },
        setAskForFingerprintDecision: function(bool) {
            window.sessionStorage.setItem('askForFingerprint', '' + bool);
            UserDetails.getAppSettings(function (err, res) {
                if (!err) {
                    var settings = res;
                    settings.showDialogAskForFingerprintDecision = bool;
                    UserDetails.updateAppSettings(settings, function (err, upd) {
                        if(err) {
                            log.error('setAskForFingerprintDecision unable to persist credentials. err -> ' + JSON.stringify(err));
                        }
                    });
                } else {
                    log.error('setAskForFingerprintDecision unable to read app settings. err -> ' + JSON.stringify(err));
                }
            });
        },
        askForFingerprint: function(res, cb){
            var opts;
            if (DeviceDetails.isAndroid()) {
                opts = {
                    title: 'Fingerabdruck Login',
                    subtitle: 'Schnell und bequem per Fingerabdruck einloggen? Ihre Anmeldedaten werden sicher verschlüsselt im Schlüsselbund Ihres Gerätes gespeichert.',
                    buttons: ['Ja', 'Später', 'Nein']
                };
            } else if (DeviceDetails.isIos()) {
                if(res && res.type==="face"){
                    opts = {
                        title: 'Face ID aktivieren',
                        subtitle: 'Schnell und bequem per Face ID einloggen? Ihre Anmeldedaten werden sicher verschlüsselt im Schlüsselbund Ihres Gerätes gespeichert.',
                        buttons: ['Ja', 'Später', 'Nein']
                    };
                } else {
                    opts = {
                        title: 'Touch ID aktivieren',
                        subtitle: 'Schnell und bequem per Fingerabdruck einloggen? Ihre Anmeldedaten werden sicher verschlüsselt im Schlüsselbund Ihres Gerätes gespeichert.',
                        buttons: ['Ja', 'Später', 'Nein']
                    };
                }
            } else {
                log.error('Plattform für Fingerprintlogin nicht erkannt.');
            }
            NativeDialogs.showActionSheet(opts, cb);
        },
        setFingerprintLoginAttemptOnStartupPerformed: function(bool) {
            window.sessionStorage.setItem('fingerprintLoginAttemptOnStartupPerformed', '' + bool);
        },
        isFingerprintLoginAttemptOnStartupPerformed: function() {
            var performed = window.sessionStorage.getItem('fingerprintLoginAttemptOnStartupPerformed');
            return (performed === 'true') ?  true : false;
        },

    },
    /**
     * Function used to display and bind app rating button on app settings.
     */
    triggerAppRatingOpenMenuEvent: function(){
        var opts = {
            title: 'Bewerten Sie unsere App',
            subtitle: 'Ihr Feedback ist uns wichtig! Gemeinsam mit Ihnen wollen wir die App stetig verbessern. Gefällt Ihnen die Cyberport App?',
            buttons: ['Geben Sie uns 5 Sterne','Melden Sie uns Ihr Feedback','Abbrechen']
        };
        var appRatingActionCallback = function (buttonIndex) {
            // ButtonIndex
            // 1: 5 Sterne,
            // 2: Feedback Formular,
            // 3: Später
            if (buttonIndex === 1) {
                if (DeviceDetails.isAndroid()) {
                    window.ADB.trackAction("setting.Apprating", {"rating":"5 Sterne Android"});
                    log.info('Debug:AppRatingEvent:Android');
                    window.location.href = Constants.appUrlAndroidStore;
                } else if (DeviceDetails.isIos()) {
                    window.ADB.trackAction("setting.Apprating", {"rating":"5 Sterne iOS"});
                    log.info('Debug:AppRatingEvent:iOS');
                    window.location.href = Constants.appUrlIosStore;
                } else {
                    log.error('Could not detect device platform / OS.');
                }
            } else if (buttonIndex === 2) {
                window.ADB.trackAction("setting.Apprating", {"rating":"Feedback geben"});
                var urlFeedbackForm = Router.getBaseUrl() + Constants.shopUrlAppRatingForm;
                log.info("Feedback Form URL: " + Router.getBaseUrl() + Constants.shopUrlAppRatingForm );
                window.location.href = urlFeedbackForm;
            } else if (buttonIndex === 3) {
                window.ADB.trackAction("setting.Apprating", {"rating":"Abbruch"});
                window.plugins.actionsheet.hide();
            }
        };
        NativeDialogs.showActionSheet(opts, appRatingActionCallback);
    },
};

/**
 * Init function handling
 */
function init() {
    UserDetails.getAppSettings(function (err, settings) {
        if(!err) {
            Deeplink.initUniversalLinks(settings);
        } else {
            log.error('onDeviceReady(): Unable to read appSettings from native storage! err -> ' + JSON.stringify(err));
        }
    });
    var currentUrl = window.location.href;
    if (Router.isAppLandingPage(currentUrl)) {
        UserDetails.checkAppSettings(function (err, res) {
            if (!err) {
                if (res === true) {
                    logExtended.debug("INIT", "Page: App landing page. handleSubsequentAppStartups ")
                    handleSubsequentAppStartups();
                } else {
                    logExtended.debug("INIT", "Page: App landing page. handleFirstAppStartup ")
                    handleFirstAppStartup();
                }
            } else {
                log.error('init()/UserDetails.checkAppSettings(): Unable to read appSettings from device on app landing page! err -> ' + JSON.stringify(err));
            }
        });
    } else {
        logExtended.debug("INIT", "Page: Shop page.")
        /* Show appsettings link in burger nav (always) and flyout when device is a tablet */
        AppSettings.showAppSettingsLink();
        navigator.splashscreen.hide();

        // this is a workaround if user changes the selected shop in app settings an then uses the "back" button of browser/app
        // selected_shop cookie and URL do not match -> redirect to homepage of shop
        if (CountryHandler.isUrlWithInvalidLocale(currentUrl)) {
            window.location.replace(Router.getBaseUrl() + Constants.shopUrlHome);
        }

        // Handle Analytics: Log successful fingerprintlogin
        if(UserDetails.isUserLoggedIn() && isFlagFingerprintLoginAttemptPerformed()){
            window.ADB.trackAction("login.Fingerprint", {"status":"Erfolgreich"});
            setFlagFingerprintloginAttemptPerformed(false);
        }

        UserDetails.getAppSettings(function (err, settings) {
            if(!err) {
                // Is called, when user clicked on logout.
                if(!app.autoLogin.isEnabledForLoggedOutUser()){
                    settings.preventAutoLoginForLoggedOutUser = true;
                    UserDetails.updateAppSettings(settings, function (err, res) {
                        if(err) {
                            log.error('init(): Unable to persist app settings. err -> ' + JSON.stringify(err));
                        }
                    });
                }

                // Current page is NOT the new checkout. Autologin/fingerprintlogin handling and interception of credentials is executed instantly after page load.
                if(!Router.isPageCheckoutNew()) {
                    app.registerFetchLoginCredentialsCallback();
                    if(!Router.isLogoutPage()){
                        handleAutoLogin(settings);
                        handleFingerprintLogin(settings, currentUrl);
                    }
                }

                // Current page IS the new checkout. Autologin/fingerprintlogin handling and interception of credentials is executed when the user reaches anmelden-adresse.html
                if(Router.isPageCheckoutNew()){

                    // event is fired when the user reaches anmelden-adresse.html
                    // event is not fired when the user enters anmelden-adresse.html directly
                    window.app.subscribe(window.app.events.FLOW_STEP_CHANGED, function([stepname]){
                        logExtended.debug("CHECKOUT", "app.events.FLOW_STEP_CHANGED", stepname);
                        if(stepname==="address"){
                            logExtended.debug("AUTOLOGIN", "Perform autologin in new checkout because stepname eq. address");
                            app.registerFetchLoginCredentialsCallback();
                            handleAutoLoginUsernameInsertionInNewCheckout(settings, currentUrl);
                            handleFingerprintLoginInNewCheckout(settings, currentUrl); // registers for fingerprintlogin
                        }
                    });
                }

                window.mobilePlatform = DeviceDetails.getMobilePlatform();
                window.senderLoginStatus = (UserDetails.isUserLoggedIn() ? "li" : "lo");
                window.selectedShop = settings.selectedShop;
            } else {
                logExtended.error(
                    'init(): Unable to read appSettings from native storage to register callbacks, handleAutoLogin, handleFingerprintLogin! ',
                    {
                        "err": Object.assign({}, err),
                        "settings": Object.assign({}, settings)
                    });
            }
        });

        initPdfShare();
        energyEfficiencyPdfToBrowser();
        initAppSettingsPage();
        BarcodeScanner.initBarcodeScanner();
        initFeedbackFormPage();
        globalHrefClickHandler();

        if((Router.isPageCheckout() || Router.isPageCheckoutNew()) && DeviceDetails.isIos()){
            window.checkoutApplePayApp = new CheckoutApplePayApp();
            window.checkoutApplePayApp.init();
        }

        if(Router.isOrderConfirmationPageNewCheckout()){
            initOrderConfirmationPageNewCheckout();
        }

    }
}

/**
 * Executes the autologin or injection of username on various pages (shop pages, wishlist, old checkout)
 * @param {} settings - local app settings.
 */
function handleAutoLogin(settings) {
    /**
     * Method to prevent further autologins by setting settings.preventAutoLoginForLoggedOutUser to TRUE
     * @param settings
     */
    const expireAutoLogin = function(settings){
        log.debug("handleAutoLogin: Autologin is expired");
        settings.preventAutoLoginForLoggedOutUser = true;
        UserDetails.updateAppSettings(settings, function (err, updatedSettings) {
            if (err) {
                log.error('handleAutoLogin isExpired: Unable to persist updated appSettings after failed autoLogin. err -> ' + JSON.stringify(err));
            }
        });
    }

    /**
     * Method to perform the autologin, when the user has valid credentials on the device and autologin activated
     * @param settings
     */
    const performAutoLogin = function(settings){
        // Might trigger autologin only on pages with header & user flyout.
        var loginContainer = getLoginComponentHeaderBarNoReload();

        // add handler to remove loading animation of login component and invalidate credentials on error.
        app.loginFailedHandler.add('errorAutoLogin', function () {
            NativeDialogs.showNotificationAlert('Login fehlgeschlagen', 'Ihr gespeichertes Passwort ist nicht mehr gültig. Bitte melden Sie sich erneut an.', 'OK');
            window.animationHelper.removeLoadingAnimation(loginContainer.find('.component.login'));
            settings.credentials.valid = false;
            UserDetails.updateAppSettings(settings, function (err, updatedSettings) {
                if (err) {
                    log.error('init: Unable to persist updated appSettings after failed autoLogin. err -> ' + JSON.stringify(err));
                }
            });
        });
        loginWithInvisibleForm(loginContainer, settings.credentials.username, settings.credentials.password);
    }

    /**
     * Method to perform the insertion of the username in a login dialog in case the user has valid credentials, but autologin is expired
     * @param settings
     */
    const performUsernameInsertion = function(settings){
        var username = settings.credentials.username;

        // Header
        var loginHeader = getLoginComponentHeaderBar();
        if (loginHeader.length !== 0) {
            loginHeader.find('form.loginForm').find('input[name="j_username"]').val(username);
        }

        // Checkout (old)
        if(Router.isPageCheckout()){
            let loginCheckout = getLoginComponentCheckoutProcess();
            if (loginCheckout.length !== 0) {
                loginCheckout.find('form.loginForm').find('input[name="j_username"]').val(username);
            }
        }

        // Wishlist
        var loginWishlist = getLoginComponentWishList();
        if (loginWishlist.length !== 0) {
            loginWishlist.find('form.loginForm').find('input[name="j_username"]').val(username);
        }
    }

    // Start the methods workflow
    if (UserDetails.isUserLoggedIn()) {
        reactivateAutoLogin(settings);
    } else {
        if(app.autoLogin.isExpired()){
            expireAutoLogin(settings);
        }
        if (!app.autoLogin.isExpired() && settings.autoLogin === true && !settings.preventAutoLoginForLoggedOutUser && settings.credentials.username && settings.credentials.password && settings.credentials.valid) {
            performAutoLogin(settings);
        } else {
            performUsernameInsertion(settings)
        }
    }
}

/**
 * Function sets the username into the login dialog in the new checkout
 * @param {} settings - local app settings.
 */
function handleAutoLoginUsernameInsertionInNewCheckout(settings, currentUrl) {
    // Action which is executed when the user sees the login box in the new checkout
    if (UserDetails.isUserLoggedIn()) {
        reactivateAutoLogin(settings);
    } else {
        if (settings.autoLogin === true && settings.credentials.valid && settings.credentials.username) {
            // Autologin not expired, user not logged in, but user logged himself out previously. When this is the case, only the saved username is inserted into the login form
            var username = settings.credentials.username;
            let loginCheckout = getLoginComponentCheckoutProcessNew();
            if (loginCheckout.length !== 0) {
                loginCheckout.find('[data-js="loginForm"]').find('input[name="j_username"]').val(username);
            }
        }
    }
}

/**
 * When the user manually logs in himself successfully, this method can re-enable the autologin.
 * @param settings
 */
function reactivateAutoLogin(settings){
    if(settings.preventAutoLoginForLoggedOutUser){
        settings.preventAutoLoginForLoggedOutUser = false;
        app.autoLogin.enableForLoggedOutUser();
        log.debug('fired: handleAutoLogin isUserLoggedIn: preventAutoLoginForLoggedOutUser false');
        UserDetails.updateAppSettings(settings, function (err, updatedSettings) {
            if (err) {
                log.error('init: Unable to persist updated appSettings. err -> ' + JSON.stringify(err));
            }
        });
    }
}

/**
 * Function handles fingerprint login.
 */
function handleFingerprintLogin(settings, currentUrl) {
    FingerprintSensor.isFingerprintScannerAvailable(function (err, res) {
        if (!err) {
            if (res.isAvailable) {
                registerEventHandlersForFingerprintLogin(settings, currentUrl);
                if (UserDetails.isUserLoggedIn()) {
                    handleAskForFingerprintDecisionDialog(settings);
                } else { // user is not logged in
                    log.debug('Fingerprint, User not logged in..');
                    if (settings.fingerprintLogin === true && settings.credentials.username && settings.credentials.password && settings.credentials.valid) {
                        if (!UserDetails.isUserLoggedIn() && (!settings.autoLogin || settings.preventAutoLoginForLoggedOutUser) && ((app.autoLogin.isExpired() && app.autoLogin.isShowFingerprintWhenExpired()) || (!app.fingerprintLogin.isFingerprintLoginAttemptOnStartupPerformed()))) {
                            log.debug('Auto Login expired. Showing Fingerprint dialog.');
                            FingerprintSensor.verifyFingerprint(function(err, fingerprintIsValid) {
                                log.debug('attempt to logging in after touchID verification');
                                app.fingerprintLogin.setFingerprintLoginAttemptOnStartupPerformed(true);
                                if (!err) {
                                    if (fingerprintIsValid === true) {
                                        var loginComponent = getLoginComponentHeaderBar();
                                        // add handler to remove loading animation of login component
                                        app.loginFailedHandler.add('errorDefaultLogin', function(){
                                            NativeDialogs.showNotificationAlert('Login fehlgeschlagen', 'Ihr gespeichertes Passwort ist nicht mehr gültig. Bitte melden Sie sich erneut an.', 'OK');
                                            window.animationHelper.removeLoadingAnimation(loginComponent.find('.component.login'));
                                        });
                                        loginWithInvisibleForm(loginComponent, settings.credentials.username, settings.credentials.password, "fingerprint");
                                    } else {
                                        window.ADB.trackAction("login.Fingerprint", {"status":"Fehlgeschlagen - Fingerprintverifizierung fehlgeschlagen"});
                                    }
                                } else {
                                    log.error('Error attempting to verify fingerprint. err -> ' + JSON.stringify(err));
                                    app.autoLogin.disableForLoggedOutUser();                                    }
                                // Only show fingerprint dialog one time
                                app.autoLogin.setShowFingerprintWhenExpired(false);
                            });
                        }
                    }
                }
            }
        } else {
            log.error('handleFingerprintLogin(): Unable to check state of fingerprint scanner. err ->' + JSON.stringify(err));
        }
    });
}

function handleFingerprintLoginInNewCheckout(settings){
    const TAG = "FINGERPRINTLOGIN-NEWCHECKOUT";
    logExtended.debug(TAG, "New checkout: attach fingerprintlogin to new checkout");

    var loginComponent = getLoginComponentCheckoutProcessNew();
    if(typeof loginComponent!=="undefined"){
        logExtended.debug(TAG, "New checkout: Loginbox found on: "+window.location.href);
        const state = {
            "isUserLoggedIn" : UserDetails.isUserLoggedIn,
            "settings.fingerprintLogin": settings.fingerprintLogin,
            "settings.credentials.username" : settings.credentials.username,
            "settings.credentials.password" : "length:"+settings.credentials.password.length,
            "settings.credentials.valid" : settings.credentials.valid
        }

        if (!UserDetails.isUserLoggedIn() && settings.fingerprintLogin && settings.credentials.username && settings.credentials.password && settings.credentials.valid) {
            logExtended.debug(TAG, 'show fingerprint in new checkout!');
            FingerprintSensor.verifyFingerprint(function (err, fingerprintIsValid) {
                logExtended.debug(TAG, 'attempt to logging in after touchID');
                if (!err) {
                    if (fingerprintIsValid === true) {
                        app.loginFailedHandler.add('errorCheckoutProcessLogin', function () {
                            // TODO: ADD error handler?
                        });
                        loginWithInvisibleFormNewCheckout(loginComponent, settings.credentials.username, settings.credentials.password, "fingerprint");
                    }
                } else {
                    logExtended.error(TAG, 'Attempt cancelled on new checkout: Error trying to verify fingerprint. err -> ', {"err": err, "state":state});
                }
            });
        } else {
            logExtended.debug(TAG, "Attempt cancelled on new checkout: User is already logged in or does not have valid credentials", {"state": state});
        }
    } else {
        logExtended.debug(TAG, "Attempt cancelled on new checkout: Loginbox undefined/not found on: "+window.location.href);
    }
}

/**
 * Function displays app settings link in burger menu and if app settings page is opened, sets values in page.
 * Function removes loading spinner in app settings page.
 */
function initAppSettingsPage(){
    if(Router.isAppSettingsPage()){
        // Button for App-Rating
        appSettingsPageUi.initAppRatingPopUpFunction();
        // Sets App Version
        appSettingsPageUi.setUiAppVersion();

        // Start App Settings
        UserDetails.getAppSettings(function (err, res) {
            if (!err) {
                var settings = res;

                // Country/Shop
                appSettingsPageUi.setUiShopSwitchCountry(settings);
                appSettingsPageUi.handleSwitchShopCountry(settings);

                // Auto Login
                appSettingsPageUi.setUiAutologin(settings);
                appSettingsPageUi.handleSwitchAutologin(settings);
                // Fingerprint
                FingerprintSensor.isFingerprintScannerAvailable(function (err, fingerPrintScanner) {
                    if (!err) {
                        appSettingsPageUi.setUiFingerprintLogin(settings, fingerPrintScanner);
                        appSettingsPageUi.handleSwitchFingerprintLogin(settings, fingerPrintScanner);
                    } else {
                        appSettingsPageUi.setUiFingerprintLogin(settings, err);
                    }
                });

                // Hide Spinner
                var appSettingsSpinner = $('#appSettingsSpinner');
                if(appSettingsSpinner){
                    appSettingsSpinner.hide();
                }
            } else {
                log.error('unable to read app settings. err -> ' + JSON.stringify(err));
            }
        });
    }
}



/**
 * Invisible form used for fingerprintlogin or autologin.
 *
 * @param {} loginComponent Cloned login form
 * @param {string} username
 * @param {string} password
 * @param {string} loginPerformedBy logintype, e.g. fingerprint. Used to set if fingerprintlogin was performed on startup.
 */
function loginWithInvisibleForm(loginComponent, username, password, loginPerformedBy) {

    if(loginPerformedBy){
        if(loginPerformedBy === "fingerprint"){
            setFlagFingerprintloginAttemptPerformed(true);
        }
    }
    UserDetails.getAppSettings(function (err, res) {
        if (!err) {
            var settings = res;
            settings.preventAutoLoginForLoggedOutUser = false;
            app.autoLogin.enableForLoggedOutUser();
            log.debug('fired: loginWithInvisibleForm: preventAutoLoginForLoggedOutUser false');

            UserDetails.updateAppSettings(settings, function (err, res) {
                if(err) {
                    log.error('loginWithInvisibleForm unable to persist credentials. err -> ' + JSON.stringify(err));
                }
            });
        } else {
            log.error('loginWithInvisibleForm unable to read app settings. err -> ' + JSON.stringify(err));
        }

        var loginComponentCloned = loginComponent.clone();
        window.login.init(loginComponentCloned);

        var loginForm = loginComponentCloned.find('form.loginForm');
        loginForm.find('input[name="j_username"]').val(username);
        loginForm.find('input[name="j_password"]').val(password);
        window.animationHelper.addLoadingAnimation(loginComponent.find('.component.login'));
        loginComponentCloned.find('form.loginForm').submit();

        handleAskForFingerprintDecisionDialog(settings);
    });
}

    /**
     * Invisible form used for fingerprintlogin or autologin.
     *
     * @param {} loginComponent Cloned login form
     * @param {string} username
     * @param {string} password
     * @param {string} loginPerformedBy logintype, e.g. fingerprint. Used to set if fingerprintlogin was performed on startup.
     */
    function loginWithInvisibleFormNewCheckout(loginComponent, username, password, loginPerformedBy) {

        if(loginPerformedBy){
            if(loginPerformedBy === "fingerprint"){
                setFlagFingerprintloginAttemptPerformed(true);
            }
        }
        UserDetails.getAppSettings(function (err, res) {
            if (!err) {
                var settings = res;
                settings.preventAutoLoginForLoggedOutUser = false;
                app.autoLogin.enableForLoggedOutUser();
                log.debug('fired: loginWithInvisibleForm: preventAutoLoginForLoggedOutUser false');

                UserDetails.updateAppSettings(settings, function (err, res) {
                    if(err) {
                        log.error('loginWithInvisibleForm unable to persist credentials. err -> ' + JSON.stringify(err));
                    }
                });
            } else {
                log.error('loginWithInvisibleForm unable to read app settings. err -> ' + JSON.stringify(err));
            }

            var loginForm = loginComponent.find('[data-js="loginForm"]');
            loginForm.find('input[name="j_username"]').val(username);

            var data = {
                cartReminderId: "",
                j_password: password,
                j_username: username,
                j_validate: "true"
            }
            window.loginHelper.login(data);

            handleAskForFingerprintDecisionDialog(settings);
        });
    }

/**
 * @returns {jQuery|HTMLElement|*}
 */
function getLoginComponentHeaderBar() {
    return $('#interaction-bar-search div.loginContainer'); // Login Komponente Startseite
}
/**
 * @returns {jQuery|HTMLElement|*}
 */
function getLoginComponentHeaderBarNoReload() {
    return $('#interaction-bar-search div.component.login'); // Login Komponente Startseite
}
/**
 * @returns {jQuery|HTMLElement|*}
 */
function getLoginComponentCheckoutProcess() {
    return $('div.loginRegistration .loginContainer');  // Login Komponente Checkout Prozess
}
/**
 * @returns {jQuery|HTMLElement|*}
 */
function getLoginComponentCheckoutProcessNew() {
    $(".component .loginForm").attr('data-js', 'loginContainer'); //TODO: remove
    return $('[data-js="loginContainer"]');  // Login Komponente New Checkout
}
/**
 * @returns {jQuery|HTMLElement|*}
 */
function getLoginComponentWishList() {
    return $('#wishlistSaveOverlay div.loginContainer'); // Login Komponente Merkzettel Speichern
}

/**
 * Used for registering fingerprintlogin event handlers.
 *
 * @param {} settings Local app settings
 * @param {string} currentUrl
 */
function registerEventHandlersForFingerprintLogin(settings, currentUrl) {
    try{
        if(app.loginFailedHandler.get()===0){
            // Add error message dialog. Will be displayed if for some reason the login with fingerprint fails.
            app.loginFailedHandler.add('errorMessageFingerpring', function(){
                NativeDialogs.showNotificationAlert('Login fehlgeschlagen', 'Eine Anmeldung ist mit diesen Benutzerdaten nicht möglich', 'OK');
            });
        }
        log.debug('registerEventHandlersForFingerprintLogin fired');
        // Login / Useraccount Flyout
        var res = $('.interaction-bar-account a.main-header-button');
        if (res.length === 1) {
            // Handler darf nur dann feuern, wenn das Flyout nicht geöffnet war (wenn es geöffnet ist, hat es zusätzlich die CSS Klasse is--open )
            res.on('click touchend', function(event) {
                event.stopPropagation();
                event.preventDefault();
                log.debug('registerEventHandlersForFingerprintLogin res.on click fired');
                if ($(this).hasClass('is--open') && !UserDetails.isUserLoggedIn()){
                    log.debug('show fingerprint...');
                    UserDetails.getAppSettings(function (err, settings) {
                        if (!err) {
                            if(settings.fingerprintLogin && settings.credentials.username && settings.credentials.password && settings.credentials.valid){
                                FingerprintSensor.verifyFingerprint(function(err, fingerprintIsValid) {
                                    log.debug('attempt to logging in after touchID verification');
                                    if (!err) {
                                        if (fingerprintIsValid === true) {
                                            var loginComponent = getLoginComponentHeaderBar();

                                            // add handler to remove loading animation of login component
                                            app.loginFailedHandler.add('errorDefaultLogin', function(){
                                                window.animationHelper.removeLoadingAnimation(loginComponent.find('.component.login'));
                                            });

                                            loginWithInvisibleForm(loginComponent, settings.credentials.username, settings.credentials.password, "fingerprint");
                                        }
                                    } else {
                                        log.error('Error trying to verify fingerprint. err -> ' + JSON.stringify(err));
                                    }
                                });
                            }
                        } else {
                            log.error('handleSubsequentAppStartups: Unable to read appSettings. error -> ' + JSON.stringify(err));
                        }
                    });
                }
            });
        }
        // Merkzettel speichern (nicht eingeloggt, daher soll TouchID beim Aufrufen der "Speichern" Funktion angezeigt werden.
        if (currentUrl.indexOf('wishlist.html') > -1) {
            var saveListLinkElem = $('.dropdownBox .save .saveList');
            if(saveListLinkElem.length === 1) {
                saveListLinkElem.on('click', function () {
                    if (!UserDetails.isUserLoggedIn() && settings.fingerprintLogin && settings.credentials.username && settings.credentials.password && settings.credentials.valid) {
                        log.debug('Show fingerprint!');
                        FingerprintSensor.verifyFingerprint(function(err, fingerprintIsValid) {
                            log.debug('attempt to logging in after touchID verification');
                            if (!err) {
                                if (fingerprintIsValid === true) {
                                    var loginComponent = getLoginComponentWishList();

                                    app.loginFailedHandler.add('errorWishlistLogin', function(){
                                        window.animationHelper.removeLoadingAnimation(loginComponent.find('.component.login'));
                                    });

                                    loginWithInvisibleForm(loginComponent, settings.credentials.username, settings.credentials.password, "fingerprint");
                                }
                            } else {
                                log.error('Error trying to verify fingerprint. err -> ' + JSON.stringify(err));
                            }
                        });
                    }
                });
            }
            // Warenkorb und Checkout Prozess (Old checkout)
        } else if (currentUrl.indexOf('tools/checkout.html') > -1 || currentUrl.indexOf('checkout.cart.html') > -1 ) {
            // Old Checkout
            var elems = document.getElementsByClassName('component checkout');
            if (elems.length === 1) {
                // Observes attribute 'data-current-view' for changes and fires fingerprint dialog if necessary
                var observer = new MutationObserver(function (mutations) {
                    mutations.forEach(function (mutation) {
                        if (mutation.type === 'attributes' && mutation.attributeName === 'data-current-view' && mutation.target.getAttribute('data-current-view') === 'account') {
                            if (!UserDetails.isUserLoggedIn() && settings.fingerprintLogin && settings.credentials.username && settings.credentials.password && settings.credentials.valid) {
                                log.debug('show fingerprint in old checkout!');

                                FingerprintSensor.verifyFingerprint(function (err, fingerprintIsValid) {
                                    log.debug('attempt to logging in after touchID');
                                    if (!err) {
                                        if (fingerprintIsValid === true) {
                                            var loginComponent = getLoginComponentCheckoutProcess();

                                            app.loginFailedHandler.add('errorCheckoutProcessLogin', function () {
                                                window.animationHelper.removeLoadingAnimation(loginComponent.find('.component.login'));
                                            });

                                            loginWithInvisibleForm(loginComponent, settings.credentials.username, settings.credentials.password, "fingerprint");
                                        }
                                    } else {
                                        log.error('Error trying to verify fingerprint. err -> ' + JSON.stringify(err));
                                    }
                                });
                            }
                        }
                    });
                });
                observer.observe(elems[0], {
                    attributes: true //configure observer to listen to attribute changes
                });
            }
            // Warenkorb und Checkout Prozess (New checkout)
        } else if (currentUrl.indexOf('anmelden-adresse.html') > -1) {
            // Fingerprintlogin is triggered in init()
        }
    } catch(err){
        logExtended.error("registerEventHandlersForFingerprintLogin", "Cant register event handlers for fingerprint login", {'err': err})
    }

}


/**
 * Creates empty app settings on first app start.
 */
function handleFirstAppStartup(){
    var settings;
    UserDetails.createAppSettings(function (err, res) {
        if (!err) {
            settings = res;
            CountryHandler.getLocaleFromDevice(function(err, locale) {
                if(!err) {
                    settings.selectedShop = CountryHandler.getSelectedShopForLocale(locale);

                    UserDetails.updateAppSettings(settings, function(err, updatedObj){
                        if(!err) {
                            handleSubsequentAppStartups();
                        } else {
                            log.error('handleFirstAppStartup: Unable to update appSettings. Error -> ' + JSON.stringify(err));
                        }
                    });
                } else {
                    log.error('handleFirstAppStartup: Unable to read locale from device. err -> ' + JSON.stringify(err));
                }
            });
        } else {
            log.error('handleFirstAppStartup: Unable to initialize appSettings. err -> ' + JSON.stringify(err));
        }
    });
    app.autoLogin.enableForLoggedOutUser();
}
/**
 * Functions tasks:
 * - Checks if autologin session timed out: If true, flag is set.
 * - Shows message when session is timed out
 * - Forwards user to to localized shop.
 */
function handleSubsequentAppStartups() {
    // Read AutoLoginTimeout from App Landing Page
    var loginTimeoutFromServer = $('#app-auto-login-timeout').attr('data-app-auto-login-timeout');
    var autoLoginTimeout = !isNaN(loginTimeoutFromServer) ? loginTimeoutFromServer : 30;

    UserDetails.getAppSettings(function (err, settings) {
        if (!err) {
            settings.autoLoginTimeout = parseInt(autoLoginTimeout);
            if (settings.lastStartup !== 0 && (new Date().getTime() - settings.lastStartup > (settings.autoLoginTimeout * 24 * 60 * 60 * 1000 ))) {
                log.info('App was not used for more than ' + settings.autoLoginTimeout + ' days. Autologin is disabled.');
                app.autoLogin.setExpired(true);
                settings.countForSessionTimeoutMessage = 1;
                if (settings.fingerprintLogin) {
                    app.autoLogin.setShowFingerprintWhenExpired(true);
                }
            } else {
                app.autoLogin.setExpired(false);
                settings.countForSessionTimeoutMessage = 0;
            }
            settings.lastStartup = new Date().getTime();

            UserDetails.updateAppSettings(settings, function (err, updatedObj) {
                if (err) {
                    log.error('handleSubsequentAppStartups: Unable to update appSettings. Error -> ' + JSON.stringify(err));
                }
                CookieHandler.handleCookieForLocalizedShop(updatedObj);
                if(deeplinkFiredFlag && deeplinkFiredUrl && !deeplinkFiredUrl.includes("cyberport.de/blog")){
                    log.debug("No redirect to the shop start page, as the App was opened via Deeplink and the Deeplink module will do the redirect within universalLinks.subscribe to the opened deeplink");
                } else {
                    log.debug("Redirect to the shop start page (DE/AT)");
                    CountryHandler.forwardToLocalizedShop(updatedObj);
                }
            });
        } else {
            log.error('handleSubsequentAppStartups: Unable to read appSettings. error -> ' + JSON.stringify(err));
        }
    });
}

/**
 * Function shows ask for fingerprint dialog and saves decision in app settings
 * @param {} userSettings Local app settings
 */
function handleAskForFingerprintDecisionDialog(userSettings){
    FingerprintSensor.isFingerprintScannerAvailable(function (err, res) {
        if (!err) {
            if (res.isAvailable) {
                var settings = userSettings;
                if ((settings.fingerprintLoginDecision === '' || (settings.fingerprintLoginDecision === 'later' && settings.showDialogAskForFingerprintDecision)) && !settings.fingerprintLogin && settings.credentials.valid && UserDetails.isUserLoggedIn()) {
                    settings.showDialogAskForFingerprintDecision = false; //disable dialog for this session
                    app.fingerprintLogin.askForFingerprint(res, function (buttonIdx) {
                        switch (buttonIdx) {
                            case 1: { // Yes
                                window.ADB.trackAction("aktivierung.Fingerprint",{"status.aktivierung.fingerprint": "aktivieren"});
                                settings.fingerprintLogin = true;
                                settings.fingerprintLoginDecision = 'yes';
                                settings.showDialogAskForFingerprintDecision = false;
                                break;
                            }
                            case 2: { // Later
                                window.ADB.trackAction("aktivierung.Fingerprint",{"status.aktivierung.fingerprint": "spaeter"});
                                //settings.fingerprintLogin = true;
                                settings.fingerprintLoginDecision = 'later';
                                settings.showDialogAskForFingerprintDecision = false;
                                break;
                            }
                            case 3: { // No
                                window.ADB.trackAction("aktivierung.Fingerprint",{"status.aktivierung.fingerprint": "nein"});
                                settings.fingerprintLogin = false;
                                settings.fingerprintLoginDecision = 'no';
                                settings.showDialogAskForFingerprintDecision = false;
                                break;
                            }
                        }
                        UserDetails.updateAppSettings(settings, function (err, res) {
                            if (err) {
                                log.error('handleAskForFingerprintLogin(): 1 Error persisting appSettings. err -> ' + JSON.stringify(err));
                            }
                        });
                    });
                }
            }
        }
    });
}

/**
 * Used to SET whether fingerprintlogin attempt was performed to enable/disable further fingerprintlogin dialogs
 * @param bool
 */
function setFlagFingerprintloginAttemptPerformed(bool) {
    window.sessionStorage.setItem('fingerprintLoginAttemptPerformed1', '' + bool);
}

/**
 * Used to GET whether fingerprintlogin attempt was performed or not
 * @returns {boolean}
 */
function isFlagFingerprintLoginAttemptPerformed() {
    var fplPerformed = window.sessionStorage.getItem('fingerprintLoginAttemptPerformed1');
    return (fplPerformed === 'true') ?  true : false;
}
/**
 * Function for filling and handling page "app settings". Handles:
 * - Switch Shop/Country
 * - Switch Autologin
 * - Switch Fingerprint login
 * - App Rating Button
 * - Sets App Version
 */
var appSettingsPageUi = {
    /**
     * Fills country switch on app settings
     * @param settings
     */
    setUiShopSwitchCountry: function(settings){
        $("#selectShopSelectBoxItOptions").remove();
        if (settings.selectedShop === 'de') {
            $('#selectShop').val('de');
            $('#selectShopSelectBoxItText').text('Deutschland');
        } else if (settings.selectedShop === 'at') {
            $('#selectShop').val('at');
            $('#selectShopSelectBoxItText').text('Österreich');
        } else {
            log.error('Unexpected value in app settings for property "selectedShop: ' + settings.selectedShop);
        }
    },
    /**
     * Handles changes on country switch on app settings
     * @param settings
     */
    handleSwitchShopCountry: function(settings){
        var opts = {
            title: 'Shop-Auswahl',
            subtitle: 'Bitte wählen Sie Ihr Land',
            buttons: ['Deutschland','Österreich', 'Abbrechen']
        };
        var appShopSwitchCallback = function (buttonIndex) {
            // ButtonIndex
            // 1: DE
            // 2: AT
            if (buttonIndex === 1) {
                settings.selectedShop = "de";
            } else if(buttonIndex===2){
                settings.selectedShop = "at";
            }
            if(buttonIndex === 1 || buttonIndex === 2){
                UserDetails.updateAppSettings(settings, function (err, res) {
                    if (err) {
                        log.error('initAppSettingsPage unable to persist settings. err -> ' + JSON.stringify(err));
                    } else {
                        window.ADB.trackAction("setting.Countryselection", {"country":settings.selectedShop});
                        CookieHandler.handleCookieForLocalizedShop(settings);
                        window.location.replace(Router.getBaseUrlForLocale(res.selectedShop) + Constants.shopUrlAppSettings);
                    }
                });
            }
        };
        $("#shopSelectBtn").on("touchend", function(e){
            e.preventDefault();
            NativeDialogs.showActionSheet(opts, appShopSwitchCallback);
        });
    },
    /**
     * Sets autologin switch on app settings page
     * @param settings
     */
    setUiAutologin: function(settings){
        if (settings.autoLogin) {
            $('#switchAutologing').attr('checked', true);
        } else if (!settings.autoLogin) {
            $('#switchAutologing').attr('checked', false);
        } else {
            log.error('Unexpected value in app settings for property "autoLogin: ' + settings.autoLogin);
        }
    },
    /**
     * Handles changes on autologin switch on app settings page
     * @param settings
     */
    handleSwitchAutologin: function(settings){
        $('#switchAutologing').change(function () {
            settings.autoLogin = !settings.autoLogin;
            UserDetails.updateAppSettings(settings, function (err, res) {
                if (err) {
                    log.error('initAppSettingsPage unable to persist autoLogin settings. err -> ' + JSON.stringify(err));
                }
                if(res){
                    if(settings.autoLogin){
                        window.ADB.trackAction("setting.Autologin", {"status.Autologin":"aktivieren"});
                        log.debug("adb settings.autoLogin: "+settings.autoLogin);
                    } else {
                        window.ADB.trackAction("setting.Autologin", {"status.Autologin":"deaktivieren"});
                        log.debug("adb settings.autoLogin: "+settings.autoLogin);
                    }
                }
            });
        });
    },
    /**
     * Sets Fingerprintlogin Switch on app settings page
     * @param settings
     * @param fingerPrintScanner
     */
    setUiFingerprintLogin: function (settings, fingerPrintScanner) {
        if (!fingerPrintScanner.isHardwareDetected) {
            $('#fingerprintBtn').parent().next().css('display', 'none');
            $('#fingerprintBtn').parent().css('display', 'none');
        } else {
            if(DeviceDetails.isIos()){
                if(fingerPrintScanner.type==="face"){
                    $('.icon-fingerprint').css('display', 'none');
                    $('.menuBtnText.ios-faceid').css('display', 'block');
                    if(!fingerPrintScanner.isAvailable){
                        $('.appMenuHelperText.copyBig.ios-faceid.isConfigured').css('display', 'block');
                    } else {
                        $('.appMenuHelperText.copyBig.ios-faceid:not(.isConfigured)').css('display', 'block');
                    }
                }
                else if(DeviceDetails.isIphoneX()){
                    $('.icon-fingerprint').css('display', 'none');
                    $('.menuBtnText.ios-faceid').css('display', 'block');
                    $('.appMenuHelperText.copyBig.ios-faceid.isConfigured').css('display', 'block');
                }
                else {
                    $('.menuBtnText.ios-fingerprint').css('display', 'block');
                    if(!fingerPrintScanner.isAvailable){
                        $('.appMenuHelperText.copyBig.ios-fingerprint.isConfigured').css('display', 'block');
                    } else {
                        $('.appMenuHelperText.copyBig.ios-fingerprint:not(.isConfigured)').css('display', 'block');
                    }
                }
            } else if(DeviceDetails.isAndroid()){
                $('.menuBtnText.android-fingerprint').css('display', 'block');
                if(!fingerPrintScanner.isAvailable){
                    $('.appMenuHelperText.copyBig.android-fingerprint.isConfigured').css('display', 'block');
                } else {
                    $('.appMenuHelperText.copyBig.android-fingerprint:not(.isConfigured)').css('display', 'block');
                }
            }
            if(settings.fingerprintLogin){
                $('#switchFingerprint').attr('checked', true);
            } else {
                $('#switchFingerprint').attr('checked', false);
            }
            if(!fingerPrintScanner.isAvailable){
                $('#fingerprintBtn').parent().next().find(".appMenuHelperText").text('Der Fingerprint Sensor muss erst in den Smartphone-Einstellungen konfiguriert werden, bevor dieses Feature genutzt werden kann!');
            }
        }
    },
    /**
     * Handles changes on fingerprintlogin switch on app settings page
     * @param settings
     * @param fingerPrintScanner
     */
    handleSwitchFingerprintLogin: function(settings, fingerPrintScanner){
        if (fingerPrintScanner.isHardwareDetected) {
            $('#switchFingerprint').change(function(){
                if($('#switchFingerprint').is(':checked')){
                    settings.fingerprintLogin = true;
                } else {
                    settings.fingerprintLogin = false;
                }
                settings.fingerprintLoginDecision = 'yes';
                UserDetails.updateAppSettings(settings, function (err, res) {
                    if(err) {
                        log.error('initAppSettingsPage/handleSwitchFingerprintLogin unable to persist fingerprintSettings. err -> ' + JSON.stringify(err));
                    } else {
                        if(res.fingerprintLogin){
                            window.ADB.trackAction("setting.Fingerprint", {"status.Fingerprint":"aktivieren"});
                            log.debug("adb res.fingerprintLogin: "+res.fingerprintLogin);
                        } else {
                            window.ADB.trackAction("setting.Fingerprint", {"status.Fingerprint":"deaktivieren"});
                            log.debug("adb res.fingerprintLogin: "+res.fingerprintLogin);
                        }

                        if(res.fingerprintLogin && !UserDetails.isUserLoggedIn()){
                            UserDetails.getAppSettings(function (err, settings) {
                                if (!err) {
                                    if(settings.fingerprintLogin && settings.credentials.username && settings.credentials.password && settings.credentials.valid){
                                        FingerprintSensor.verifyFingerprint(function(err, fingerprintIsValid) {
                                            log.debug('attempt to logging in after touchID verification');
                                            if (!err) {
                                                if (fingerprintIsValid === true) {
                                                    var loginComponent = getLoginComponentHeaderBar();

                                                    // add handler to remove loading animation of login component
                                                    app.loginFailedHandler.add('errorDefaultLogin', function(){
                                                        window.animationHelper.removeLoadingAnimation(loginComponent.find('.component.login'));
                                                    });

                                                    loginWithInvisibleForm(loginComponent, settings.credentials.username, settings.credentials.password, "fingerprint");
                                                }
                                            } else {
                                                log.error('AppSettings FingerprintLogin: Error trying to verify fingerprint. err -> ' + JSON.stringify(err));
                                            }
                                        });
                                    }
                                } else {
                                    log.error('AppSettings FingerprintLogin: Unable to read appSettings. error -> ' + JSON.stringify(err));
                                }
                            });
                        }
                    }
                });
            });
        }
    },
    /**
     * Inserts app version on app settings page
     */
    setUiAppVersion: function(){
        var tmpAppVersion;
        if (DeviceDetails.isAndroid()) {
            tmpAppVersion = Constants.appVersionAndroid + ' (Android)';
        } else if (DeviceDetails.isIos()) {
            tmpAppVersion = Constants.appVersionIos + ' (iOS)';
        } else {
            tmpAppVersion = '(' + Constants.appVersionAndroid + ' / ' + Constants.appVersionIos + ')';
        }
        $('.appSettingsMenu').after('<p style="padding-left:12px;">App-Version: ' + tmpAppVersion + '</p>');
    },
    /**
     * Binds function on app rating button on app settings page
     */
    initAppRatingPopUpFunction: function (){
        // Bind app rating click event
        var appRatingButton = $('#rateAppBtn');
        if (appRatingButton.length === 1) {
            $(appRatingButton).on('click', function(event){
                app.triggerAppRatingOpenMenuEvent();
            });
        }
    }
}
try{
    /*if(Constants.appHostUrl!="https://app.cyberport.de"){
        window.testUtils = testUtils;
        var logs = [];
        window.appLogs=logs;
    }*/
    console.log(Constants.appHostUrl);
    app.initialize();
} catch(err){
    console.log("APP-ERROR", err);
}
})($);

