import {debugLogger as log} from '../logger/log';

/**
 * Class is handling the integration of Adjust to the App.
 */
export default class AdjustHandler {

    /**
     * Check if the User gave tracking consent
     * @returns {boolean}
     */
    static adjustTrackingAllowed(){
        try{
            var allConsents = JSON.parse(localStorage.getItem("vendorConsents"));
            if(allConsents){
                var adjustVendorGrant = allConsents.consentedVendors.find(function(vendor) {
                    if (vendor._id === "5e68dbdd69e7a93e0b259030") {
                        return true;
                    } else if (vendor.name === "Adjust") {
                        return true;
                    } else {
                        return false;
                    }
                });
                if(adjustVendorGrant && adjustVendorGrant._id){
                    var isAdjustMarketingGranted = allConsents.grants[adjustVendorGrant._id].purposeGrants['5f902f9c62ad5b2a96b1a36c']; // Grant = Marketing Grant
                    if(isAdjustMarketingGranted){
                        log.debug("Adjust: Consent given");
                        return true;
                    } else {
                        log.debug("Adjust: Consent not given");
                        return false;
                    }
                } else {
                    this.deactivateAdjust();
                }
            } else {
                this.deactivateAdjust();
            }
        } catch(e) {
            log.debug("Adjust: Error: "+e);
            this.deactivateAdjust();
            return false;
        }
    }

    /**
     * Initializes Adjust with config properties
     */
    static init(){
        this.assignAppFunctionsToWindowObject();
        Adjust.requestTrackingAuthorizationWithCompletionHandler(function(status) {
            switch (status) {
                case 0:
                    // ATTrackingManagerAuthorizationStatusNotDetermined case
                    break;
                case 1:
                    // ATTrackingManagerAuthorizationStatusRestricted case
                    break;
                case 2:
                    // ATTrackingManagerAuthorizationStatusDenied case
                    break;
                case 3:
                    // ATTrackingManagerAuthorizationStatusAuthorized case
                    break;
            }
        });
        Adjust.isEnabled(function(isEnabled){
            log.debug("Adjust: Enable status: "+isEnabled);
        })
        if(this.adjustTrackingAllowed()===true){
            try {
                var appToken = '<%= appAdjustAppToken %>';

                var appAdjustEnvironment = '<%= appAdjustEnvironment %>';
                if(appAdjustEnvironment==="AdjustConfig.EnvironmentProduction"){
                    appAdjustEnvironment = AdjustConfig.EnvironmentProduction;
                } else {
                    appAdjustEnvironment =  AdjustConfig.EnvironmentSandbox;
                }

                var adjustConfig = new AdjustConfig(appToken, appAdjustEnvironment);
                adjustConfig.setLogLevel(AdjustConfig.LogLevelVerbose); // LogLevelInfo or LogLevelVerbose
                adjustConfig.setAttributionCallbackListener(function(attribution) {
                    // Printing all attribution properties.
                    log.debug("Attribution changed!");
                    log.debug(attribution.trackerToken);
                    log.debug(attribution.trackerName);
                    log.debug(attribution.network);
                    log.debug(attribution.campaign);
                    log.debug(attribution.adgroup);
                    log.debug(attribution.creative);
                    log.debug(attribution.clickLabel);
                    log.debug(attribution.adid);
                });
                adjustConfig.setEventTrackingSucceededCallbackListener(function(eventSuccess) {
                    // Printing all event success properties.
                    log.debug("Event tracking succeeded!");
                    log.debug(eventSuccess.message);
                    log.debug(eventSuccess.timestamp);
                    log.debug(eventSuccess.eventToken);
                    log.debug(eventSuccess.callbackId);
                    log.debug(eventSuccess.adid);
                    log.debug(eventSuccess.jsonResponse);
                });

                adjustConfig.setEventTrackingFailedCallbackListener(function(eventFailure) {
                    // Printing all event failure properties.
                    log.debug("Event tracking failed!");
                    log.debug(eventFailure.message);
                    log.debug(eventFailure.timestamp);
                    log.debug(eventFailure.eventToken);
                    log.debug(eventFailure.callbackId);
                    log.debug(eventFailure.adid);
                    log.debug(eventFailure.willRetry);
                    log.debug(eventFailure.jsonResponse);
                });

                adjustConfig.setSessionTrackingSucceededCallbackListener(function(sessionSuccess) {
                    // Printing all session success properties.
                    log.debug("Session tracking succeeded!");
                    log.debug(sessionSuccess.message);
                    log.debug(sessionSuccess.timestamp);
                    log.debug(sessionSuccess.adid);
                    log.debug(sessionSuccess.jsonResponse);
                });
                adjustConfig.setSessionTrackingFailedCallbackListener(function(sessionFailure) {
                    // Printing all session failure properties.
                    log.debug("Session tracking failed!");
                    log.debug(sessionFailure.message);
                    log.debug(sessionFailure.timestamp);
                    log.debug(sessionFailure.adid);
                    log.debug(sessionFailure.willRetry);
                    log.debug(sessionFailure.jsonResponse);
                });
                adjustConfig.setDelayStart(5.0);
                Adjust.create(adjustConfig);
                Adjust.setEnabled(true);
                log.debug("Adjust: Initialization: successful");
            } catch (e) {
                log.debug("Adjust: Initialization: error");
                log.debug(e);
            }
        } else {
            this.deactivateAdjust();
        }
    }

    /**
     * Deactivates Adjust tracking in case the user removed the consent
     */
    static deactivateAdjust(){
        Adjust.isEnabled(function(isEnabled){
            if(isEnabled){
                Adjust.setEnabled(false);
            }
        })
    }

    /**
     * Binds specific tracking functions to the window so that they can be used by the Adobe Analytics Layer in the Webshop frontend
     */
    static assignAppFunctionsToWindowObject(){
        if(typeof(window.cpApp)=="undefined"){
            window.cpApp = {};
        }
        window.cpApp.adjust = {
            trackBarcodescannerEvent : AdjustHandler.trackBarcodescannerEvent
        };
        //window.cpApp.adjust.trackBarcodescannerEvent = AdjustHandler.trackBarcodescannerEvent;
        window.cpApp.adjust.trackRegistrationEvent = AdjustHandler.trackRegistrationEvent;
        window.cpApp.adjust.trackPurchaseEvent = AdjustHandler.trackPurchaseEvent;
        window.cpApp.adjust.adjustTrackingAllowed = AdjustHandler.adjustTrackingAllowed;

    }

    /**
     * Tracks barcodescanning events
     */
    static trackBarcodescannerEvent(){
        if(this.adjustTrackingAllowed()){
            log.debug("adjust event triggered: "+ "trackBarcodescannerEvent start");
            let eventTokenBarcodescanner = '<%= appAdjustEventTokenBarcodescanner %>';
            var adjustEvent = new AdjustEvent(eventTokenBarcodescanner);
            Adjust.trackEvent(adjustEvent);
            log.debug("adjust event triggered: "+ "trackBarcodescannerEvent end");
        }
    }

    /**
     * Tracks registration events
     */
    static trackRegistrationEvent(){
        if(this.adjustTrackingAllowed()){
            log.debug("adjust event triggered: "+ "trackRegistrationEvent start");
            let eventTokenRegistration = '<%= appAdjustEventTokenRegistration %>';
            var adjustEvent = new AdjustEvent(eventTokenRegistration);
            Adjust.trackEvent(adjustEvent);
            log.debug("adjust event triggered: "+ "trackRegistrationEvent end");
        }
    }

    /**
     * Tracks purchases
     * @param revenue
     */
    static trackPurchaseEvent(revenue){
        try{
            if(this.adjustTrackingAllowed()){
                log.debug("adjust event triggered: "+ "trackPurchaseEvent start");
                log.debug("adjust event triggered: "+ "trackPurchaseEvent revenue:"+revenue);
                let eventTokenPurchase = "<%= appAdjustEventTokenPurchase %>";
                var adjustEvent = new AdjustEvent(eventTokenPurchase);
                if(!isNaN(revenue)){
                    adjustEvent.setRevenue(revenue, "EUR");
                }
                Adjust.trackEvent(adjustEvent);
                log.debug("adjust event triggered: "+ "trackPurchaseEvent end");
            }
        } catch (e) {
            log.debug("Adjust.trackPurchaseEvent error: "+e);
        }
    }
}
