import {Router} from "./router";
import CookieHandler from "./cookieHandler";
import {debugLogger as log} from "../logger/log";
import Constants from "../constants";

/**
 * The CountryHandler is used to determine the locale (DE/AT) of the User.
 * The locale is used to determine whether show the German or Austrian webshop.
 */
export default class CountryHandler{

    /**
     * Forwards user to country/shop specified in app settings
     * @param settings
     */
    static forwardToLocalizedShop(settings) {
        log.debug("forwardToLocalizedShop with settings: "+JSON.stringify(settings));
        let locale = settings.selectedShop;
        if (locale) {
            let url = Router.getBaseUrlForLocale(locale) + Constants.shopUrlHome;
            log.debug("forwardToLocalizedShop: locale set; redirect to: "+url);
            window.location.replace(url);
        } else {
            let url = Router.getBaseUrlForLocale('de') + Constants.shopUrlHome;
            log.debug("forwardToLocalizedShop: locale NOT set; redirect to: "+url);
            window.location.replace(url);
        }
    }

    /**
     * Respects only the region from a locale string (i.e. function will return AT for en-AT)
     * @param locale
     * @returns {*}
     */
    static getSelectedShopForLocale(locale) {
        let region = locale.split('-');
        if (region.length === 2) {
            if (region[1].toLowerCase() === 'at') {
                return 'at';
            }
        }
        return 'de';
    }

    /**
     * Reads the locale string from device settings (e.g. de-DE, en-US, de-AT, ... )
     * @param cb
     */
    static getLocaleFromDevice(cb) {
        navigator.globalization.getLocaleName(function(locale) {
                cb(null, locale.value);
            },
            function(err){
                cb(err, null);
            });
    }

    /**
     *
     * @param url
     * @returns {boolean}
     */
    static isUrlWithInvalidLocale(url) {
        let selectedShop = CookieHandler.readCookie('selected_shop');
        let regex = /webshop30_(\S\S)/g;

        let match = regex.exec(url);
        log.info('isUrlWithInvalidLocale(): url: '+url);
        log.info('isUrlWithInvalidLocale(): match: '+match);
        if(match){
            let localeFromUrl = match[1];

            if(localeFromUrl.toLowerCase() !== selectedShop.toLowerCase()){
                log.debug("isUrlWithInvalidLocale() match / return true");
                return true;
            } else {
                log.debug("isUrlWithInvalidLocale() match / return false");
                return false;
            }
        }
        // If (austrian) user gets redirected into root directory (URL without country path, root shows german shop),
        // user gets redirected into austrian AT shop.
        if(window.location.pathname === "/"){
            if(!(selectedShop.toLowerCase()==="de")){
                log.debug("isUrlWithInvalidLocale() pathname AT / return true");
                return true;
            }
        }
    }
}
