import Route from '../model/route';
import {Router} from "./router";
import {debugLogger as log} from '../logger/log';

/**
 * Module handles deep linking: When the user clicks a cyberport.de link e.g. on mail client, he is asked if he wants to open the link in the Cyberport app.
 * This function receives the input link (e.g. from intent) and the user settings (important for country de/at).
 * Input link can contain cyberport.de with or without subdomains, pdp, root, category ...
 * Input link and user settings are then used to build and return the app domain link to which the user should be redirected inside the app.
 * E.g.
 * From  https://cyberport.de?DEEP=1H62-020                 to  https://app.cyberport.de/content/webshop30_de/de.html?DEEP=1H62-020
 * From  https://www.cyberport.de/apple-und-zubehoer.html   to  https://app.cyberport.de/content/webshop30_de/de/apple-und-zubehoer.html
 */
var currentRoute = new Route(window.location.href);
var transformedDeeplinkUrl = null;
var deeplinkFiredFlag = false;
var deeplinkFiredUrl;
var res = null;
var FLAG_BLOG_INAPPBROWSER = "flag-blog-inappbrowser";

class Deeplink {
    /**
     * Function for deep linking: Deep links open external URLs from browser or mail client in the app
     */
    static initUniversalLinks(settings){
        Deeplink.setSettings(settings);
        Deeplink.openBlogInInappBrowserHandler();



        universalLinks.subscribe('cpShopMain', function (eventData) {
            log.debug("DEEPLINK: FIRED for:"+eventData.url+" on: "+window.location.href);

            /**
             * Flag to prevent automatic redirection to the Shop start page (https://app.cyberport.de/content/webshop30_de/de.html)
             * in case the App is opened due to click on a specific cyberport.de deeplink.
             */
            deeplinkFiredFlag = true;
            deeplinkFiredUrl = eventData.url;

            var eventDataObject = eventData;
            Deeplink.getEventDataUrl(eventDataObject.url)
            .then(url => {
                log.debug("DEEPLINKING Deeplink.getEventDataUrl() url:"+url);
                /**
                 * Case 1: Blog URL
                 * If the deeplink is the cyberport.de/blog url, open the url in the inappbrowser above the webview .
                 * 1.1 If the App and therefore a page!=app.html in window.location is already opened, do no redirect.
                 * 1.2 If the App is started new, and the window.location is the blank app.html, redirect the user to the start page of the web shop before
                 * open the inappbrowser.
                 *
                 * Case 2: mail.cyberport.de Newsletter Link
                 * If the deeplink is a link like https://click.mail.cyberport.de/*, open the URL in the webview as it might the page to
                 * deactivate/activate newsletter registrations
                 *
                 * Case 3: Other URL
                 * Open the deeplink URL in the webview.
                 */

                if(url.includes("cyberport.de/blog")){
                    // Current page
                    if(window.location.href.includes("app.html")){
                        // Case 1.1: Blank app start page, set flag and redirect to shop start page
                        window.sessionStorage.setItem(FLAG_BLOG_INAPPBROWSER, 'true');
                        var shopRootUrl = Router.getBaseUrlWithoutCountryPath();
                        log.debug("DEEPLINKING REDIRECTION TO cyberport.de/blog:" + shopRootUrl);
                    } else {
                        // Case 1.2: Page with content already opened, open inappbrowser above the content
                        var ref = cordova.InAppBrowser.open(url, '_blank', 'location=no,footer=yes,shouldPauseOnSuspend=yes,fullscreen=no');
                    }
                } else if(url.includes("mail.cyberport.de")){
                    log.debug("DEEPLINKING REDIRECTION TO mail.cyberport.de:" + url.toString());
                    window.location.replace(url.toString());
                } else {
                    transformedDeeplinkUrl = Deeplink.transformInputUrlToRedirectableUrl(url);
                    if (transformedDeeplinkUrl != null) {
                        log.debug("DEEPLINKING: !isAppLandingPage fired for: " + window.location.href);
                        log.debug("DEEPLINKING REDIRECTION TO:" + transformedDeeplinkUrl);
                    }
                    window.location.replace(transformedDeeplinkUrl);
                }
            })
            .catch(error => {
                log.debug(error);
            });
        });
    }

    static setSettings(settings){
        res = settings;
    }

    static openBlogInInappBrowserHandler(){
        var flag = window.sessionStorage.getItem("flag-blog-inappbrowser");
        log.debug("DEEPLINKING flag:"+flag+"for:"+window.location.href);
        if(window.location.href.includes("de.html")){
            if(window.sessionStorage.getItem(FLAG_BLOG_INAPPBROWSER)){
                if(window.sessionStorage.getItem(FLAG_BLOG_INAPPBROWSER)==="true"){
                    var url = "https://cyberport.de/blog"
                    log.debug("DEEPLINKING InAppBrowser.open:"+url);
                    var ref = cordova.InAppBrowser.open(url, '_blank', 'location=no,footer=yes,shouldPauseOnSuspend=yes,fullscreen=no');
                }
                window.sessionStorage.removeItem(FLAG_BLOG_INAPPBROWSER);
            }
        }
    }

    /**
     * Fetches the URL to redirect from the 302 Response Headers
     * @param routeInput
     * @returns {Promise<any>}
     */
    static async getEventDataUrl(routeInput){
        var url = routeInput;
        // If the URL contains a richrelevance URL:
        // There might be a 302/303 redirect chain -> newsletter -> richrelevance -> richrelevance -> cyberport
        // To receive the final cyberport link, it is looped several times on the redirect urls
        var i=0;
        while(i<5){
            if(url.includes("richrelevance.com")){
                url = url.replace("http://","https://"); // Added because richrelevance redirect URL might contain http instead of https
                url = await this.fetchRedirectUrlFrom302ResponseHeader(new Route(url))
                i = i++;
            } else if(url.includes("parcel-api.cyberport.de") ){
                url = url.replace("https://","http://");  // Added because parcel-api is only reachable on http, not on https.
                url = await this.fetchRedirectUrlFrom302ResponseHeader(new Route(url))
                i = i++;
            } else {
                break;
            }
        }
        return url;
    }

    static async fetchRedirectUrlFrom302ResponseHeader(routeInput){
        var url = await new Promise((resolve,reject)=> {
            log.debug("DEEPLINK: NEWSLETTER LINK");
            // Case: Deeplink comes from newsletter and is a 302 redirect. (http://click.mail.cyberport.de/)
            // This block fetches the 302 ResponseHeader.location, which is then used as deeplink.
            cordova.plugin.http.setFollowRedirect(false);
            cordova.plugin.http.clearCookies();
            const options = {method: 'head', timeout: 10};
            cordova.plugin.http.sendRequest(routeInput.getFullRoute(), options,
                function (responseSuccess) {
                    log.debug("DEEPLINK: Newsletter location response success:" + responseSuccess.url);
                    resolve(responseSuccess.url);
                }, function (responseError) {
                    log.debug("DEEPLINK: Newsletter location response failed for:" + routeInput.getFullRoute());
                    log.debug(JSON.stringify(responseError));
                    var status = ""+responseError.status;
                    if (status.startsWith("3")) {
                        // e.g. 300, 302, 303...
                        resolve(responseError.headers.location);
                    } else {
                        resolve("https://app.cyberport.de");
                    }
                });
        });
        return url;
    }

    /**
     * Function receives the input link and then creates and returns the app link for redirection.
     * @param {string} url - Deeplinking plugins provides eventData.url which contains the URL which should be opened
     * @param {object} settings - Contains user settings on device (important for redirection to DE or AT)
     * @returns {string} url - Contains the eventData.url mapped to app domains
     */
    static transformInputUrlToRedirectableUrl(url){
        log.debug("DEEPLINK.transformInputUrlToRedirectableUrl URL::"+JSON.stringify(url));
        var routeInput = new Route(url);
        var routeOutput = null;
        if(routeInput.isPathPdpDeeplink()){
            /* Case: PDP Deeplink, e.g.: https://cyberport.de?DEEP=1H62-020 or https://app.cyberport.de?DEEP=1H62-020 */
            var deepSearchString = routeInput.getSearchString().replace("/","");   // returns ?DEEP=1H62-020
            routeOutput = new Route(Router.getBaseUrlForLocale(res.selectedShop) + "de.html" + deepSearchString);
            log.debug("DEEPLINK output (for pdp): "+routeInput.getFullRoute());
        } else if(routeInput.isDirectoryRoot()){
            var deepSearchString = routeInput.getSearchString().replace("/","");
            /* Case: Root, e.g. cyberport.de/ or app.cyberport.de/ */
            routeOutput = new Route(Router.getBaseUrlForLocale(res.selectedShop) + "de.html"+deepSearchString);
            log.debug("DEEPLINK output (for root directory): "+routeInput.getFullRoute());
        } else if(routeInput.isPathAppStartpage()){
            /* Case: Startpage de.html on subdomain, e.g.: https://app.cyberport.de/content/webshop30_de/de.html */
            routeOutput = new Route(Router.getBaseUrlForLocale(res.selectedShop) + "de.html");
            log.debug("DEEPLINK output (for startpage de.html on subdomain): "+routeInput.getFullRoute());
        }  else {
            /* Case: All other paths.
            /* E.g. https://www.cyberport.de/apple-und-zubehoer.html or https://app.cyberport.de/content/webshop30_de/de/apple-und-zubehoer.html
            /* If input path contains webshop30_ path structure, this is removed and afterwards added depending on user settings
             */
            var pathname = routeInput.getPathname();
            var searchString = routeInput.getSearchString();
            pathname = pathname.replace("/content/webshop30_de/de", "");
            pathname = pathname.replace("/content/webshop30_at/de", "");
            routeOutput = new Route(Router.getBaseUrlForLocale(res.selectedShop) + "de" + pathname + searchString);
            log.debug("DEEPLINK output (other parts): pathname:"+pathname);
            log.debug("DEEPLINK output (other parts): searchString:"+searchString);
            log.debug("DEEPLINK output (other parts): "+routeInput.getFullRoute());
        }
        return routeOutput.getFullRoute().replace(".pdp", "/pdp");
    }
}
export {Deeplink, transformedDeeplinkUrl, deeplinkFiredFlag, deeplinkFiredUrl};
