import {debugLogger as log} from '../logger/log';

/**
 * Dialogs can be used to show the user native dialogs above the WebView.
 * Examples: Ask the user if he wants to use the fingerprint login.
 * @type {{showConfirmationDialog: showConfirmationDialog, showActionSheet: showActionSheet, showNotificationAlert: showNotificationAlert, showActionSheetWithCancel: showActionSheetWithCancel}}
 */
var Dialogs = (function () {
    /**
     * Shows an action sheet with an option to cancel the dialog.
     * @param opts
     * {
     *      title : string,     // optional
     *      subtitle : string,  // optional
     *      buttons : []
     * }
     * @param cb Callback function. Callback gets the index of clicked button as parameter.
     */
    function showActionSheetWithCancel(opts, cb) {
        opts.cancel = true;
        showActionSheet(opts, cb);
    }

    /**
     * Shows an action sheet.
     * @param opts
     * {
     *      title : string,     // optional
     *      subtitle : string,  // optional
     *      buttons : []
     * }
     * @param cb Callback function. Callback gets the index of clicked button as parameter.
     */
    function showActionSheet(opts, cb) {
        var options = {
            androidTheme: window.plugins.actionsheet.ANDROID_THEMES.THEME_DEVICE_DEFAULT_LIGHT, // material design
            buttonLabels: opts.buttons,
            //addDestructiveButtonWithLabel : 'Delete it',
            destructiveButtonLast: true // you can choose where the destructive button is shown
        };
        if (opts.cancel) {
            options.addCancelButtonWithLabel = 'Abbrechen';
            options.androidEnableCancelButton = true;
        }
        if (opts.title) {
            options.title = opts.title;
        }
        if (opts.subtitle) {
            options.subtitle = opts.subtitle; // supported on iOS only
        }
        window.plugins.actionsheet.show(options, cb);
    }


    /**
     * Shows a confirmation dialog with optional title, optional message and buttons. Callback gets the index of clicked button.
     * @param opts
     * {
     *      title : string,     // optional
     *      message : string,   // optional
     *      buttons : [],
     * }
     * @param cb Callback function
     */
    function showConfirmationDialog(opts, cb) {
        var message = opts.message ? opts.message : '';
        var title = opts.title ? opts.title : '';
        navigator.notification.confirm(message, cb, title, opts.buttons);
    }


    /**
     * Shows a native notification alert
     * @param title Window title
     * @param message Message to display
     * @param buttonName Name of the button to confirm message
     * @param cb Callback function that is executed when dialog is confirmed
     */
    function showNotificationAlert(title, message, buttonName, cb) {
        navigator.notification.alert(message, cb, title, buttonName);
    }


    // ++++++++++++++++++++++++++++++++ Public methods ++++++++++++++++++++++++++++++++
    return {
        showActionSheetWithCancel,
        showActionSheet,
        showConfirmationDialog,
        showNotificationAlert
    };
})();
export {Dialogs};
