import DeviceDetails from "./deviceDetails";
import {debugLogger as log} from "../logger/log";

export default class FingerprintSensor {

    /**
     * Returns the status of the fingerprint sensor. Result:
     * {
     *  isAvailable: boolean
     *  isHardwareDetected:	boolean
     *  hasEnrolledFingerprints: boolean
     *  type: For Apple devices: 'face' on iPhone X, 'touch' on other apple devices; Not set for Android
     * }
     * @param cb
     */
    static isFingerprintScannerAvailable(cb){
        if (DeviceDetails.isAndroid()) {
            window.FingerprintAuth.isAvailable(function (result) {
                //log.debug('Fingerprint available? res -> ' + JSON.stringify(result));
                cb(null, result);
            }, function(error) {
                //log.error('error checking Fingerprint -> ' + JSON.stringify(error));
                cb(error, null);
            });
        } else if (DeviceDetails.isIos()) {
            window.plugins.touchid.isAvailable(function(type) {
                //log.info('Fingerprint available!');
                var result = {};
                result.isAvailable = true;
                result.isHardwareDetected = true;
                result.hasEnrolledFingerprints = true;
                if(type){
                    result.type = type;
                }
                cb(null, result);
            }, function(res) {
                var result = {};
                // Status Codes iOS: Keine Finger registriert: -7, kein TouchID verbaut: -6
                if (res.code === -7) {
                    result.isAvailable = false;
                    result.isHardwareDetected = true;
                    result.hasEnrolledFingerprints = false;
                    cb(null, result);
                } else if (res.code === -6){
                    result.isAvailable = false;
                    result.isHardwareDetected = false;
                    result.hasEnrolledFingerprints = false;
                    cb(null, result);
                } else {
                    // TODO maybe use a common error Object for iOS and Android
                    cb(res, null);
                }
            });
        } else {
            cb(new Error('Unsupported platform. Only iOS and Android are supported.'), null);
        }
    }

    /**
     * Function verifies fingerprint
     * @param {}
     */
    static verifyFingerprint(cb) {
        var errorObj = {};
        errorObj.isDialogCanceled = false;
        errorObj.code = 0;
        errorObj.message = '';

        if (DeviceDetails.isAndroid()) {
            var encryptConfig = {
                clientId: 'de.cyberport.mobile.aem.app',
                //username: 'currentUser',
                //password: 'currentUserPassword',
                locale: 'de',
                disableBackup: true, // TODO Disable PIN entry screen since it is not working currently. See https://github.com/mjwheatley/cordova-plugin-android-fingerprint-auth/issues/85
                dialogTitle: 'Einloggen mit Fingerabdruck',
                dialogMessage: 'Scannen Sie Ihren Fingerabdruck, um auf Kundenkonto, Warenkorb und die Merkliste zuzugreifen.',
            };

            var successCallbackAndroid = function (result) {
                log.info('successCallbackAndroid(): ' + JSON.stringify(result));
                if (result.withFingerprint) {
                    log.info('Successfully encrypted credentials.');
                    log.info('Encrypted credentials: ' + result.token);
                    cb(null, true);
                } else if (result.withBackup) {
                    log.info('Authenticated with backup password');
                    cb(null, true);
                }
            };

            var errorCallbackAndroid = function (error) {
                if (error === window.FingerprintAuth.ERRORS.FINGERPRINT_CANCELLED) {
                    log.info('FingerprintAuth Dialog Cancelled!');
                    window.ADB.trackAction("login.Fingerprint", {"status":"Fehlgeschlagen - Abbruch durch User"});
                    errorObj.isDialogCanceled = true;
                    errorObj.code = -1;
                    errorObj.message = error;
                    cb(errorObj, false);
                } else {
                    log.error('FingerprintAuth Error1: ' + error);
                    window.ADB.trackAction("login.Fingerprint", {"status":"Fehlgeschlagen"});
                    errorObj.isDialogCanceled = false;
                    errorObj.code = -1;
                    errorObj.message = error;
                    cb(errorObj, false);
                }
            };

            window.FingerprintAuth.encrypt(encryptConfig, successCallbackAndroid, errorCallbackAndroid);

        } else if (DeviceDetails.isIos()) {
            var successCallbackIos = function () {
                cb(null, true);
            };

            var errorCallbackIos = function (res) {
                // res -> {
                //          localizedDescription: string,
                //          code: int // bei -128 wurde der dialog abgebrochen
                //         }
                if (res.code === -128) {
                    log.info('FingerprintAuth Dialog Cancelled!');
                    window.ADB.trackAction("login.Fingerprint", {"status":"Fehlgeschlagen - Abbruch durch User"});
                    errorObj.isDialogCanceled = true;
                    errorObj.code = res.code;
                    errorObj.message = res.localizedDescription;
                    cb(false, errorObj);
                } else {
                    window.ADB.trackAction("login.Fingerprint", {"status":"Fehlgeschlagen"});
                    errorObj.isDialogCanceled = false;
                    errorObj.code = res.code;
                    errorObj.message = res.localizedDescription;
                    log.error('FingerprintAuth error: ' + JSON.stringify(errorObj));
                    cb(false, errorObj);
                }
            };

            // TODO maybe add the check mentioned here: https://github.com/EddyVerbruggen/cordova-plugin-touch-id#security
            window.plugins.touchid.verifyFingerprint('Scannen Sie Ihren Fingerabdruck, um auf Kundenkonto, Warenkorb und die Merkliste zuzugreifen.', successCallbackIos, errorCallbackIos);
        } else {
            log.error('Fingerprint Scanner is not supported on Your current device platform.');
        }
    }

}
